﻿using AutoMapper;
using ContactManager.Models;
using Microsoft.AspNet.Mvc;
using Microsoft.Data.Entity;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace ContactManager.Controllers
{
    [Route("api/[controller]")]
    public class PersonTypesController : Controller
    {
        [FromServices]
        public Data.ContactContext Context { get; set; }

        [HttpGet]
        [Route("")]
        public async Task<IActionResult> GetAll()
        {
            var data = await Context.PersonTypes.OrderBy(one => one.TypeName).ToListAsync();
            return Ok(data.Select(Mapper.Map<PersonType>));
        }

        [HttpGet]
        [Route("{id}")]
        public async Task<IActionResult> GetOne(int id)
        {
            var data = await Context.PersonTypes.FirstOrDefaultAsync(one => one.PersonTypeId == id);
            if(data == null)
            {
                return HttpNotFound();
            }
            return Ok(Mapper.Map<PersonType>(data));
        }

        [HttpPost]
        [Route("")]
        public async Task<IActionResult> Insert([FromBody] PersonType personType)
        {
            if (ModelState.IsValid)
            {
                var target = Mapper.Map<Data.PersonType>(personType);
                Context.Entry(target).State = EntityState.Added;
                await Context.SaveChangesAsync();
                return Created("", Mapper.Map<PersonType>(target));
            }
            return HttpBadRequest(ModelState);
        }

        [HttpPut]
        [Route("")]
        public async Task<IActionResult> Update([FromBody] PersonType personType)
        {
            if (ModelState.IsValid) 
            {
                var target = Mapper.Map<Data.PersonType>(personType);
                Context.Update(target, GraphBehavior.IncludeDependents);
                try
                {
                    await Context.SaveChangesAsync();
                }
                catch (DbUpdateConcurrencyException)
                {
                    return HttpNotFound(personType);
                }
                
                return new NoContentResult();
            }
            return HttpBadRequest(ModelState);
        }

        [HttpDelete]
        [Route("{id}")]
        public async Task<IActionResult> Delete(int id)
        {
            var type = new Data.PersonType { PersonTypeId = id };
            Context.Entry(type).State = EntityState.Deleted;
            try
            {
                await Context.SaveChangesAsync();
            }
            catch (DbUpdateConcurrencyException)
            {
                return HttpNotFound(id);
            }

            return new NoContentResult();
        }

    }
}