﻿using AutoMapper;
using ContactManager.Models;
using Microsoft.AspNet.Mvc;
using Microsoft.Data.Entity;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace ContactManager.Controllers
{
    [Route("api/[controller]")]
    public class PersonsController : Controller
    {
        [FromServices]
        public Data.ContactContext Context { get; set; }

        [HttpGet]
        [Route("")]
        public async Task<IActionResult> GetAll()
        {
            var data = await Context.People
                .OrderBy(one => one.LastName)
                .Select(one => new PersonInfo
                {
                    PersonId = one.PersonId,
                    FirstName = one.FirstName,
                    LastName = one.LastName
                }).ToListAsync();
            return Ok(data);
        }

        [HttpGet]
        [Route("{id}")]
        public async Task<IActionResult> GetOne(int id)
        {
            var data = await Context.People.FirstOrDefaultAsync(one => one.PersonId == id);
            if (data == null)
            {
                return HttpNotFound();
            }
            return Ok(Mapper.Map<Person>(data));
        }

        [HttpPost]
        [Route("")]
        public async Task<IActionResult> Insert([FromBody] Person person)
        {
            if (ModelState.IsValid)
            {
                var target = Mapper.Map<Data.Person>(person);
                Context.Entry(target).State = EntityState.Added;
                await Context.SaveChangesAsync();
                return Created("", Mapper.Map<Person>(target));
            }
            return HttpBadRequest(ModelState);
        }

        [HttpPut]
        [Route("")]
        public async Task<IActionResult> Update([FromBody] Person person)
        {
            if (ModelState.IsValid)
            {
                var target = Mapper.Map<Data.Person>(person);
                Context.Update(target, GraphBehavior.IncludeDependents);
                try
                {
                    await Context.SaveChangesAsync();
                }
                catch (DbUpdateConcurrencyException)
                {
                    return HttpNotFound(person);
                }

                return new NoContentResult();
            }
            return HttpBadRequest(ModelState);
        }

        [HttpDelete]
        [Route("{id}")]
        public async Task<IActionResult> Delete(int id)
        {
            var type = new Data.Person { PersonId = id };
            Context.Entry(type).State = EntityState.Deleted;
            try
            {
                await Context.SaveChangesAsync();
            }
            catch (DbUpdateConcurrencyException)
            {
                return HttpNotFound(id);
            }
            return new NoContentResult();
        }

    }
}